import isPropValid from '@emotion/is-prop-valid';
import { createElement, Fragment, forwardRef } from 'react';
import { withCSSContext } from '@emotion/core';
import { getRegisteredStyles, insertStyles, shouldSerializeToReactTree } from '@emotion/utils';
import { serializeStyles } from '@emotion/serialize';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var testOmitPropsOnStringTag = isPropValid;
var testOmitPropsOnComponent = function testOmitPropsOnComponent(key) {
  return key !== 'theme' && key !== 'innerRef';
};
var testAlwaysTrue = function testAlwaysTrue() {
  return true;
};
var omitAssign = function omitAssign(testFn, target) {
  var i = 2;
  var length = arguments.length;

  for (; i < length; i++) {
    var source = arguments[i];

    var _key = void 0;

    for (_key in source) {
      if (testFn(_key)) {
        target[_key] = source[_key];
      }
    }
  }

  return target;
};

var createStyled = function createStyled(tag, options) {
  if (process.env.NODE_ENV !== 'production') {
    if (tag === undefined) {
      throw new Error('You are trying to create a styled element with an undefined component.\nYou may have forgotten to import it.');
    }
  }

  var identifierName;

  if (options !== undefined) {
    identifierName = options.label;
  }

  var isReal = tag.__emotion_real === tag;
  var baseTag = isReal && tag.__emotion_base || tag;
  var omitFn = typeof baseTag === 'string' && baseTag.charAt(0) === baseTag.charAt(0).toLowerCase() ? testOmitPropsOnStringTag : testOmitPropsOnComponent;
  return function () {
    var args = arguments;
    var styles = isReal && tag.__emotion_styles !== undefined ? tag.__emotion_styles.slice(0) : [];

    if (identifierName !== undefined) {
      styles.push("label:" + identifierName + ";");
    }

    if (args[0] == null || args[0].raw === undefined) {
      styles.push.apply(styles, args);
    } else {
      styles.push(args[0][0]);
      var len = args.length;
      var i = 1;

      for (; i < len; i++) {
        styles.push(args[i], args[0][i]);
      }
    }

    var Styled = withCSSContext(function (props, context) {
      var className = '';
      var classInterpolations = [];
      var mergedProps = omitAssign(testAlwaysTrue, {}, props, {
        theme: props.theme || context.theme
      });

      if (typeof props.className === 'string') {
        className += getRegisteredStyles(context.registered, classInterpolations, props.className);
      }

      var serialized = serializeStyles.call(mergedProps, styles.concat(classInterpolations));
      var rules = insertStyles(context, serialized);
      className += serialized.cls;
      var ele = createElement(baseTag, omitAssign(omitFn, {}, props, {
        className: className,
        ref: props.innerRef
      }));

      if (shouldSerializeToReactTree && rules !== undefined) {
        return createElement(Fragment, null, createElement("style", {
          "data-emotion-ssr": serialized.name,
          dangerouslySetInnerHTML: {
            __html: rules
          }
        }), ele);
      }

      return ele;
    });
    Styled.displayName = identifierName !== undefined ? identifierName : "Styled(" + (typeof baseTag === 'string' ? baseTag : baseTag.displayName || baseTag.name || 'Component') + ")"; // $FlowFixMe

    var FinalStyled = forwardRef(function (props, ref) {
      if (ref === null) {
        // this avoids creating a new object if there's no ref
        return createElement(Styled, props);
      }

      return createElement(Styled, _extends({}, props, {
        innerRef: ref
      }));
    });
    FinalStyled.__emotion_real = FinalStyled;
    FinalStyled.__emotion_base = baseTag;
    FinalStyled.__emotion_styles = styles;

    FinalStyled.withComponent = function (nextTag, nextOptions) {
      return createStyled(nextTag, nextOptions !== undefined ? _extends({}, options, nextOptions) : options).apply(void 0, styles);
    };

    return FinalStyled;
  };
};

export default createStyled;
//# sourceMappingURL=index.es.js.map
